/**
 * @fileoverview This rule enforces correct usage of getI18NValue method
 * @author Aravindh T
 */

let i18nKeys = []
let commentsToIgnoreCheck = []

let aliasMethodName = ''
const ERROR_MESSAGE_NO_HARDCODING = 'Do not hardcode content inside getI18NValue method. Use I18N keys instead'
const ERROR_MESSAGE_NO_CONCATINATING = 'Do not concat getI18NValue method. Use a single key'
const ERROR_MESSAGE_NO_PARAMETER = 'Do not pass getI18NValue or its alias name as parameter to getI18NValue method'

module.exports={
  meta:{
    type: "problem",
    docs: {
        description: "This rule enforces correct usage of keys inside getI18NValue method"
    }
  },
    create: function(context){	
      const excludes = context.options[0]
      i18nKeys = excludes.i18nKeys  
      commentsToIgnoreCheck = excludes.commentsToIgnoreCheck   	             
        return{           
          VariableDeclarator:function(node){
            aliasMethodName = ''
            if(node.init !== null && node.init.callee) {
              if(node.init.callee.type === 'MemberExpression' && node.init.callee.property.name === 'getI18NValue'){      
              aliasMethodName = node.id.name              
          }
              else if(node.init.callee.type === 'Identifier' && node.init.callee.name =='getI18NValue' ){
            	aliasMethodName = node.id.name
            }          
        }
        if(node.init !== null && node.init.type === 'ConditionalExpression'){
          if(node.init.consequent.type === 'CallExpression' && node.init.alternate.type === 'CallExpression'){
            if(node.init.consequent.callee && node.init.alternate.callee){
              if((node.init.consequent.callee.type === 'MemberExpression' && node.init.consequent.callee.property.name === 'getI18NValue')
            || (node.init.alternate.callee.type === 'MemberExpression' && node.init.alternate.callee.property.name === 'getI18NValue')){
              aliasMethodName = node.id.name
            }
            else if((node.init.consequent.callee.type === 'Identifier' && node.init.consequent.callee.name === 'getI18NValue' )
            ||(node.init.alternate.callee.type === 'Identifier' && node.init.alternate.callee.name === 'getI18NValue' )){
              aliasMethodName = node.id.name
            }
            }
            
          }
        }
    },
    CallExpression: function(node){  
      //variables for reporting errors
      let parameters = []
      let isGetI18NValueMethodPassedAsParameter = false
      let isI18NNotNeeded = false
      let isStringHardCodedAsArgument = false

      //variables for checking comments
      let ancestors = context.getAncestors()
      let correctAncestors = ancestors.find(node => node.type === 'Program')
      let commentedCode = correctAncestors.comments.filter(node =>  commentsToIgnoreCheck.includes(node.value.trim().toLowerCase()))
      let functionsStartsAt = node.loc.start.line
      let functionsEndsAt = node.loc.end.line
      

      let ancestorWithComment = ancestors.find(node => node.type === 'JSXElement' 
      && node.parent.type === 'ReturnStatement')

      //checking if method name is getI18NValue
		  if((node.callee.type === 'MemberExpression'
          && (node.callee.property.name ==='getI18NValue' || node.callee.property.name === aliasMethodName))
          || (node.callee.type ==='Identifier'
          && (node.callee.name ==='getI18NValue' || node.callee.name === aliasMethodName))){
        	parameters = node.arguments
        }
        parameters.forEach((node) =>
                           {
        	switch(node.type){
              case 'Identifier':
              	isGetI18NValueMethodPassedAsParameter = identifierResolver(node)
                break
              
              case 'ArrayExpression':
                isGetI18NValueMethodPassedAsParameter = arrayResolver(node)
                break
              
              case 'CallExpression':
                isGetI18NValueMethodPassedAsParameter = functionResolver(node)
                break
              
              case 'Literal':
                isStringHardCodedAsArgument = stringResolver(node) 
                break
            }
        })
        //checking if comments exists
        if((isGetI18NValueMethodPassedAsParameter && commentedCode)
        || (isStringHardCodedAsArgument && commentedCode)){            
                isI18NNotNeeded  = commentedCode.some(node => node.loc.start.line === functionsStartsAt
                                                        || node.loc.start.line === functionsEndsAt
                                                        || (node.loc.start.line > functionsStartsAt 
                                                        && node.loc.start.line < functionsEndsAt)) 
                if(!isI18NNotNeeded && ancestorWithComment){
                    isI18NNotNeeded = commentedCode.some(node=>node.loc.start.line === ancestorWithComment.parent.loc.start.line
                                                    || node.loc.start.line === ancestorWithComment.parent.loc.end.line
                                                    )
                }                        
        }
        //reporting the errors       
        if(isGetI18NValueMethodPassedAsParameter && !isI18NNotNeeded){
      	context.report({
        	node:node,
          message: ERROR_MESSAGE_NO_PARAMETER
        })
      }
        if(isStringHardCodedAsArgument && !isI18NNotNeeded){
          context.report({
            node:node,
            message: ERROR_MESSAGE_NO_HARDCODING
          })
        }
    },
    BinaryExpression:function(node){
      //variables for checking various cases
      let getI18NValueMethodAsleftToken = false
      let getI18NValueMethodAsRightToken = false
      let aliasMethodAsLeftToken = false
      let aliasMethodAsRightToken = false
      let stringAsLeftToken = false
      let stringAsRightToken = false
      let isI18NNotNeeded = false

      //variables for checking comments
      let ancestors = context.getAncestors()
      let correctAncestors = ancestors.find(node => node.type === 'Program')
      let commentedCode = correctAncestors.comments.filter(node => node.value.trim().toLowerCase() === 'no i18n')
      let concatinationStartsAt = node.loc.start.line
      let concatinationEndsAt = node.loc.end.line

        if(node.operator === '+'){
            if(node.left.type === 'CallExpression' || node.right.type === 'CallExpression'){
                 if(node.left.callee){
                   if((node.left.callee.type === 'Identifier' && (node.left.callee.name === 'getI18NValue' || node.left.callee.name === aliasMethodName))
                    ||(node.left.callee.type ==='MemberExpression' && node.left.callee.property.name === 'getI18NValue')){
                      getI18NValueMethodAsleftToken = true
                  }
                 }
                 if(node.right.callee){
                   if((node.right.callee.type === 'Identifier' && (node.right.callee.name === 'getI18NValue' || node.right.callee.name === aliasMethodName))
                    ||(node.right.callee.type ==='MemberExpression' && node.right.callee.property.name === 'getI18NValue')){
                      getI18NValueMethodAsRightToken = true
                  }
                 }                   	                     	
              }
              if(node.left.type === 'Identifier' || node.right.type === 'Identifier'){
                  if(node.left.type === 'Identifier' && node.left.name === aliasMethodName){
                      aliasMethodAsLeftToken = true
                  }
                  if(node.right.type === 'Identifier' && node.right.name === aliasMethodName){
                      aliasMethodAsRightToken = true
                  }
              }
              if(getI18NValueMethodAsleftToken || getI18NValueMethodAsRightToken
                || aliasMethodAsLeftToken || aliasMethodAsRightToken){
                  if(node.left.type === 'Literal'){
                    stringAsLeftToken = true
                  }
                  if(node.right.type === 'Literal'){
                    stringAsRightToken = true
                }              
          }  
      //checking if comments exist
      isI18NNotNeeded = commentedCode.some(node => node.loc.start.line === concatinationStartsAt
                                                    || node.loc.start.line === concatinationEndsAt
                                                    || (node.loc.start.line > concatinationStartsAt
                                                    && node.loc.start.line < concatinationEndsAt))
          

      //reporting the errors for various cases    
      if(((getI18NValueMethodAsleftToken && getI18NValueMethodAsRightToken)
      || (aliasMethodAsLeftToken && aliasMethodAsRightToken)
      || (getI18NValueMethodAsleftToken && aliasMethodAsRightToken)
      || (getI18NValueMethodAsRightToken && aliasMethodAsLeftToken)
      || (stringAsLeftToken && stringAsRightToken)
      || (getI18NValueMethodAsleftToken && stringAsRightToken)
      || (getI18NValueMethodAsRightToken && stringAsLeftToken)
      || (aliasMethodAsLeftToken && stringAsRightToken)
      || (aliasMethodAsRightToken && stringAsLeftToken)) && !isI18NNotNeeded){
        context.report({
            node:node,
            message: ERROR_MESSAGE_NO_CONCATINATING
            })
      }   
    }         
        }
    }
}
}

//helper functions to  check if argument is getI18NValue method
function identifierResolver(node){
  if(node.name === aliasMethodName){
  return true 
}}
function arrayResolver(node){
  let isGetI18NValueAnArrayElement = false
  isGetI18NValueAnArrayElement = node.elements.some(element => element.value === aliasMethodName)
  return isGetI18NValueAnArrayElement
}
function functionResolver(node){
  let isGetI18NValueAParameter = false
  if(node.callee.name === aliasMethodName || node.callee.name === 'getI18NValue'){
  	isGetI18NValueAParameter = true
  }
  return isGetI18NValueAParameter
}
function stringResolver(node){
  let keyFormat = /^(\w+\.)+\w+$/ 
  let sentenceFormat = /(\w+\s+){1,}\w+/     
  let isValidKey = false
  let isValidKeyFormat = false
  let isSentenceFormat = false

  isValidKeyFormat = keyFormat.test(node.value)
  
  isSentenceFormat = sentenceFormat.test(node.value)
  if(!isValidKeyFormat && isSentenceFormat){
    isValidKey = i18nKeys.some(key => key === node.value)
  }
  
  if(isSentenceFormat && !isValidKey && !isValidKeyFormat){
    return true
  }
  else{
    return false
  }
}