/**
 * @fileoverview Do not hardcode content. Use appropriate i18n keys
 * @author Aravindh T
 */
'use strict';

const {RuleTester} = require('eslint')
var rule = require('../../../lib/rules/no-hardcoding')


RuleTester.setDefaultConfig({
    env :{
        browser : true,
        commonjs: true,
        es6: true,
        node: true
      },
      parserOptions: {
        ecmaVersion: 6,
        ecmaFeatures: {
          jsx: true
        },
        sourceType: 'module'
      }
})

var ruleTester = new RuleTester();

ruleTester.run('no-hardcoding',rule,{
    valid : [
        {
            code: `<Container className="test string"/>`
        },
        {
            code: 'dataId: string'
        },
        {
            code: `<Container text='hardcoded string' /> //no i18n`
        },
        {
            code: `let index = findIndex('firstElement')`
        },
        {
            code: `<svg title='incorrect hardcoded string' />`
        },
        {
            code: `let i18nedValue = getI18NValue('Product Information')`
        },
        {
            code: `
                {
                    displayLabel: 'hardcoded string' //ignore i18n start
                    dataId: 'string'
                }
                {
                    displayLabel: 'hardcoded string'
                    dataId: 'string'
                }
                {
                    displayLabel: 'hardcoded string'
                    dataId: 'string'
                }
                {
                    displayLabel: 'hardcoded string'
                    dataId: 'string'
                }
                {
                    displayLabel: 'hardcoded string' //ignore i18n end
                    dataId: 'string'
                }
            `
        }       
    ],
    invalid : [
        {
            code: `let test = "test variable"`,
            errors: [
                {
                message: 'Do not hardcode content. Use I18N key instead'
                }
            ]            
        },
        {
            code: `<Container displayLabel='hardcoded string' />`,
            errors:[
                {
                    message: 'Do not hardcode content. Use I18N key instead'
                }
            ]
        },
        {
            code:`obj: 'hardcoded string'`,
            errors:[
                {
                    message: 'Do not hardcode content. Use I18N key instead'
                }
            ]
        }    
    ]
}
);

//console.log("All tests passed!")
